-- SRP2012a

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

use WORK.ALU_PKG.all;

entity SRP is
port(Clock   : in  std_logic;
     Reset_b : in  std_logic;
	  gout0   : out std_logic;
     gout1   : out std_logic;
     gout2   : out std_logic;
     gout3   : out std_logic;
     gout4   : out std_logic );	  
end entity SRP;

architecture RTL of SRP is
  -- instruction read only memory IROM
    component IROM
	port (Add   : in  std_logic_vector (5 downto 0);
	      Dout  : out std_logic_vector (31 downto 0) );
    end component IROM;
  -- data random access memory DRAM
    component DRAM
	port (Add   : in  std_logic_vector (5 downto 0);
	      Clock : in  std_logic;
	      WE    : in  std_logic;
	      Din   : in  std_logic_vector (31 downto 0);
	      Dout  : out std_logic_vector (31 downto 0) );
    end component DRAM;
  -- minimips
    component MINIPROC
	port (Clock    : in  std_logic;
	      Reset_b  : in  std_logic;
	      Iadd     : out std_logic_vector (5 downto 0);
	      Inst     : in  std_logic_vector (31 downto 0);
	      Dadd     : out std_logic_vector (5 downto 0);
	      WE       : out std_logic;
	      Wtdata   : out std_logic_vector (31 downto 0);
	      Rddata   : in  std_logic_vector (31 downto 0) );
    end component MINIPROC;
  -- wires between entity
    signal  Iadd   : std_logic_vector (5 downto 0);
    signal  Inst   : std_logic_vector (31 downto 0);
    signal  Dadd   : std_logic_vector (5 downto 0);
    signal  WE     : std_logic;
    signal  Wtdata : std_logic_vector (31 downto 0);
    signal  Rddata : std_logic_vector (31 downto 0);
    
begin

-- IROM
ROM: IROM port map (Iadd,Inst);

-- DRAM
RAM: DRAM port map (Dadd,Clock,WE,Wtdata,Rddata);

-- MINI PROCESSOR
MPU: MINIPROC port map (Clock,Reset_b,Iadd,Inst,Dadd,WE,Wtdata,Rddata);

-- 5 REGS
FIVE_REGS: process(Clock, Reset_b) begin
	 if (Reset_b ='0') then
	   gout0 <= '0'; 
		gout1 <= '0';
		gout2 <= '0';
		gout3 <= '0';
		gout4 <= '0';
	 elsif rising_edge(Clock) then
	   if (Dadd = "111111" and WE = '1') then
	     gout0 <= Wtdata(0); 
		  gout1 <= Wtdata(1);
		  gout2 <= Wtdata(2);
		  gout3 <= Wtdata(3);
		  gout4 <= Wtdata(4);
      end if;		
	 end if;
end process FIVE_REGS;
  
end architecture RTL;